@@ -3,7 +3,6 @@
 
 import copy
 import collections
-import warnings
 from functools import total_ordering
 
 from trytond.model import fields
@@ -347,42 +346,34 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
         super(Model, self).__init__()
         if id is not None:
             id = int(id)
-        self.__dict__['id'] = id
-        self._values = None
-        parent_values = {}
-        for name, value in kwargs.iteritems():
-            if not name.startswith('_parent_'):
-                setattr(self, name, value)
-            else:
-                parent_values[name] = value
-        for name, value in parent_values.iteritems():
-            parent_name, field = name.split('.', 1)
-            parent_name = parent_name[8:]  # Strip '_parent_'
-            parent = getattr(self, parent_name, None)
-            if parent is not None:
-                setattr(parent, field, value)
-            else:
-                setattr(self, parent_name, {field: value})
-        self._init_values = self._values.copy() if self._values else None
+        self._id = id
+        if kwargs:
+            self._values = {}
+            parent_values = {}
+            for name, value in kwargs.iteritems():
+                if not name.startswith('_parent_'):
+                    setattr(self, name, value)
+                else:
+                    parent_values[name] = value
+            for name, value in parent_values.iteritems():
+                parent_name, field = name.split('.', 1)
+                parent_name = parent_name[8:]  # Strip '_parent_'
+                parent = getattr(self, parent_name, None)
+                if parent is not None:
+                    setattr(parent, field, value)
+                else:
+                    setattr(self, parent_name, {field: value})
+            self._init_values = self._values.copy()
+        else:
+            self._values = None
+            self._init_values = None
 
     def __getattr__(self, name):
-        if name == 'id':
-            return self.__dict__['id']
-        elif self._values and name in self._values:
-            return self._values.get(name)
-        raise AttributeError("'%s' Model has no attribute '%s': %s"
-            % (self.__name__, name, self._values))
-
-    def __setattr__(self, name, value):
-        if name == 'id':
-            self.__dict__['id'] = value
-            return
-        super(Model, self).__setattr__(name, value)
-
-    def __getitem__(self, name):
-        warnings.warn('Use __getattr__ instead of __getitem__',
-            DeprecationWarning, stacklevel=2)
-        return getattr(self, name)
+        try:
+            return self._values[name]
+        except (KeyError, TypeError):
+            raise AttributeError("'%s' Model has no attribute '%s': %s"
+                % (self.__name__, name, self._values))
 
     def __contains__(self, name):
         return name in self._fields
